/*
 * SWMappletContext.java
 *
 * Created on 10 de octubre de 2004, 20:33
 */

package org.jtgl.micro.impl.superwaba;

import org.jtgl.core.*;
import org.jtgl.image.*;
import org.jtgl.input.*;
import org.jtgl.impl.superwaba.*;
import org.jtgl.micro.*;

import waba.fx.*;
import waba.ui.*;
import waba.sys.Settings;

/**
 *
 * @author  Manuel
 */
public class SWMappletContext extends MainWindow implements MappletContext{
    private SWGraphics gc;    
    private Image dblBuffer;
    private ISurface ssscreen;
    private Graphics g0;
    private Mapplet currentMapplet;
    private boolean ctxDblBuffer;
    private static final String BMP_EXTENSION   =   ".bmp";
    
    /** Creates a new instance of SWMappletContext */
    public SWMappletContext() {
        flicker = false;
    }

    public void runMapplet(Mapplet mapplet) {
        currentMapplet = mapplet;
        try {
            currentMapplet.initialize(this);
        }catch(Exception ex){
            handleException(currentMapplet, ex,"Error initializing");
        }
    }    
    
    public JTGLImage createJTGLImage(String src) throws JTGLException {
        if(src == null) return null;        
        String locaseSrc = src.toLowerCase();
        if(!locaseSrc.endsWith(BMP_EXTENSION)){            
            int extension_index = locaseSrc.indexOf('.');
            src = src.substring(0,extension_index)+BMP_EXTENSION;            
        }
        Image image = new Image(src);
        return new SWImage(image);
    }
    
    public JTGLImage createJTGLImage(JTGLImage image) {
        return image;
    }
    
    public JTGLImage createJTGLImage(int width, int height) {
        Image image = new Image(width,height);
        return new SWImage(image);
    }
    
    public JTGLImage createJTGLImage(byte[] imageData, int imageOffset, int imageLength) {
        //TODO: offset 
        Image image = new Image(imageData);
        return new SWImage(image);        
    }
    
    public void delay(Mapplet mapplet, int ms) {
        try{
            Thread.currentThread().sleep(ms);
        }catch(InterruptedException ex){
        }        
    }
    
    public Object get(Mapplet mapplet, String property) {
        //TODO
        return null;
    }
    
    public ImageUtil getImageUtil() {    return new DefaultImageUtil(this);    }
    
    public String[] getAvailableInputControls(Mapplet mapplet) {
        //TODO
        return null;
    }
    
    public int getDisplayColors(Mapplet mapplet) {        
        return Settings.maxColors;
    }
    
    public InputControl getInputControl(Mapplet mapplet, String inputControlClassName) {
        //TODO
        return null;
    }
    
    public JTGLGraphics getJTGLGraphics(Mapplet mapplet) {
        if(gc == null){
            Graphics g = dblBuffer.getGraphics();
            //g.setBackColor(255,255,255);
            //g.clearScreen();           
            gc = new SWGraphics(g,this);
            gc.init();
        }
        return gc;
    }
    
    public void flushGraphics(Mapplet mapplet){
        if(gc != null)
            gc.flush();
    }
    
    public int getWidth(Mapplet mapplet){
        return getSize().width(); //Whole screen width
    }

    public int getHeight(Mapplet mapplet){
        return getSize().height(); //Whole screen height
    }
    
    public void handleException(Mapplet mapplet, Exception ex, String title) {
        //TODO
        debugMsg("Exception:"+ex);
    }
    
    public void notifyKill(Mapplet mapplet) {
        try {
            mapplet.kill(true);
        }catch(JTGLException ex){
            handleException(mapplet,ex,null);
            delay(mapplet, 2000);
        }        
    }
    
    public void debugMsg(Object obj) {
        System.out.println(obj);
    }
    
    
    //
    //MainwWindow methods:
    //
    public void onStart(){
        debugMsg("Initializing SWMappletContext");
        debugMsg("Size:"+getSize());
        setDoubleBuffer(true);
        ctxDblBuffer = false;
        dblBuffer = getOffScreen();
        if(dblBuffer == null)
            try{
                dblBuffer = new Image(getSize().width(),getSize().height());
                ctxDblBuffer = true;
            }catch(Throwable t){
                exit(-1);
            }
        ssscreen = dblBuffer;
        g0 = createGraphics();
        
        if(currentMapplet != null)
            currentMapplet.start();
        //repaintNow();
    }
    
    public void onExit(){
        if(dblBuffer != null)
            dblBuffer.free();
    }
    /*
    public void onPaint(Graphics g){
        System.out.println("onPaint "+dblBuffer);
        g.setColor(0,0,0);
        //if(ctxDblBuffer && dblBuffer != null){
        if(dblBuffer != null){
            g.copyScreen(dblBuffer,0,0,height);        
        }
        g.drawText("onPaint",50,50);
    }
     */
    public void repaintNow(){        
        g0.copyScreen(ssscreen,0,0, height);
        super.repaintNow();
    }
}
